#![feature(proc_macro_hygiene, decl_macro)]

             extern crate clap;
             extern crate handlebars;
             extern crate ldap3;
#[macro_use] extern crate rocket;
             extern crate rocket_contrib;
#[macro_use] extern crate serde_derive;
#[macro_use] extern crate serde_json;

mod api;
mod config;
mod exit;
mod r#static;

use clap::{Arg, App};
use rocket::config::{Config, Environment};

use crate::config::load_config;
use crate::exit::exit_with_error;

const VERSION: &str = "1.0";
const DEFAULT_CONFIG_FILE_PATH: &str = "/etc/webldappasswd/config.json";

fn main() {
    let matches = App::new("WebLDAPPasswd")
        .version(VERSION)
        .arg(Arg::with_name("config")
             .short("c")
             .long("config")
             .value_name("CONFIG_FILE_PATH")
             .help("Custom configuration file")
             .takes_value(true))
        .get_matches();

    let config_file_path = matches
        .value_of("config")
        .unwrap_or(DEFAULT_CONFIG_FILE_PATH);

    let config = load_config(config_file_path);

    let rocket_config_builder = Config::build(Environment::Production)
        .address(&config.host)
        .port(config.port);

    let rocket_config = match rocket_config_builder.finalize() {
        Ok(config) => config,
        Err(e) => exit_with_error(&format!("Bad host address ({}) in configuration file: {}", config.host, e)),
    };

    rocket::custom(rocket_config)
        .mount("/", routes![r#static::index,
                            r#static::legal,
                            r#static::css,
                            r#static::js,
                            r#static::checkmark,
                            r#static::cross,
                            r#static::hourglass,
                            api::update])
        .manage(config)
        .launch();
}
